// Reflection Mapped Decorations Shader
// 
// Copyright Outerspace Software, all rights reserved

string description = "This effect creates shiny decorations. The first texture must be a reflection map, the second a transparent colour map.";

float4x4	World;
float4x4	ViewProjection;
float4x4	WorldInverseTranspose;
float4		LightPos;
float4		EyePos;

float4		AmbientCol;
float4		DiffuseCol;
float4		SpecularCol;
float		Power = 100;

float4		TextureResolution0;
float4		TextureResolution1;
float4		UVBounds0;
float4		UVBounds1;

float4 Prop0
<
string UIName="Reflection Rotation";
float UIMin=-3.14159;
float UIMax=3.14159;
float3 UIDefault=float3(0,0,0);
int UISliders=3;
float UIScale=57.29577;
bool UIInteger=false;
int UIWidget=0;
> = float4(0,0,0,0);

texture MyTexture0;
sampler MyTextureSampler0 = sampler_state
{
   Texture = <MyTexture0>;
   MinFilter = Linear;
   MagFilter = Linear;
   MipFilter = Linear;
};

texture MyTexture1;
sampler MyTextureSampler1 = sampler_state
{
   Texture = <MyTexture1>;
   MinFilter = Linear;
   MagFilter = Linear;
   MipFilter = Linear;
};

struct VS_INPUT{
	float4 position:	POSITION;
	float3 normal:		NORMAL;
	float3 diffuse:		COLOR;
	float2 tex:		TEXCOORD0;
};

struct VS_OUTPUT{
	float4 position:	POSITION;
	float2 tex:		TEXCOORD0;
	float4 modelpos:	TEXCOORD1;
	float3 normal:		TEXCOORD2;	
};

struct PS_OUTPUT{
    float4 color:		COLOR;
};

float2 Normal2UVBounds(float2 inUV, float4 inUVBounds){
	inUV.x = inUVBounds.x + inUV.x*(inUVBounds.y-inUVBounds.x);
	inUV.y = inUVBounds.z + inUV.y*(inUVBounds.w-inUVBounds.z);
	return inUV;
}

float2 UVBounds2Normal(float2 inUV, float4 inUVBounds){
	inUV.x = (inUV.x-inUVBounds.x)/(inUVBounds.y-inUVBounds.x);
	inUV.y = (inUV.y-inUVBounds.z)/(inUVBounds.w-inUVBounds.z);
	return inUV;
}

float3 RotateReflection(float3 a){
	if(Prop0.x==0 && Prop0.y==0 && Prop0.z==0) return a;

	float sa=sin(Prop0.x);
	float ca=cos(Prop0.x);
	float sb=sin(Prop0.y);
	float cb=cos(Prop0.y);
	float sg=sin(Prop0.z);
	float cg=cos(Prop0.z);

	float3 b;

	b.x=a.x*ca-a.z*sa;
	b.y=a.y;
	b.z=a.x*sa+a.z*ca;

	a.x=b.x;
	a.y=b.y*cb-b.z*sb;
	a.z=b.y*sb+b.z*cb;

	b.x=a.x*cg-a.y*sg;
	b.y=a.x*sg+a.y*cg;
	b.z=a.z;

	return b;
}

VS_OUTPUT VS(VS_INPUT input){
	VS_OUTPUT output;      
	
	float4 Pos=mul(input.position, World);
	output.position=mul(Pos, ViewProjection);
	
	output.tex = input.tex;
	output.modelpos = Pos;
	output.normal = mul(input.normal,WorldInverseTranspose);
	return output;
}

PS_OUTPUT PS(VS_OUTPUT input){
	PS_OUTPUT output;

	float3 NormalVector		= normalize(input.normal);
	float3 LightVector		= normalize(LightPos - input.modelpos);
	float3 EyeVector		= normalize(EyePos   - input.modelpos);
	float3 LightReflectionVector	= -reflect(LightVector,NormalVector);


	//ambient component
	float4 PhongColour=AmbientCol;

	//diffuse component
	PhongColour += DiffuseCol * saturate(dot(NormalVector, LightVector));

	//specular component
	if(Power<=0) PhongColour += SpecularCol;
	if(Power>0)  PhongColour += SpecularCol * pow(saturate(dot(EyeVector, LightReflectionVector)),Power);


	float4 ReflectionColour=float4(1,1,1,1);	
	if(TextureResolution0.x>0){

		float3 ReflectionVector = reflect(EyeVector,NormalVector);

		ReflectionVector=RotateReflection(ReflectionVector);

		ReflectionVector.z=abs(ReflectionVector.z);

		ReflectionVector.z+=1;
		float f=2*length(ReflectionVector);
		float2 uv = float2(0.5,0.5) + ReflectionVector.xy/f;

		uv=Normal2UVBounds(uv,UVBounds0);
	
		ReflectionColour=tex2D(MyTextureSampler0, uv);
		ReflectionColour.w *= DiffuseCol.w;
	}

	//2 textures
	if(TextureResolution0.x>0 && TextureResolution1.x>0){

		float2 uv = UVBounds2Normal(input.tex, UVBounds0);
		uv = Normal2UVBounds(uv, UVBounds1);

		float a=tex2D(MyTextureSampler1, uv).w;

		output.color = (1-a)*PhongColour + a*ReflectionColour;
	}
	
	//Alleen reflectionmap
	if(TextureResolution0.x>0 && TextureResolution1.x<=0){
		output.color = ReflectionColour;
	}

	//Geen textures
	if(TextureResolution0.x<=0 && TextureResolution1.x<=0){
		output.color = PhongColour;
	}

	return output;
}

technique Shader
{
    pass P0
    {
        VertexShader = compile vs_3_0 VS();
        PixelShader  = compile ps_3_0 PS();
    }
}

